@@ -205,8 +205,8 @@ class Resources(Command):
         dest = self.j(self.RESOURCES, 'template-functions.json')
         function_dict = {}
         import inspect
-        from calibre.utils.formatter_functions import all_builtin_functions
-        for obj in all_builtin_functions:
+        from calibre.utils.formatter_functions import formatter_functions
+        for obj in formatter_functions.get_builtins().values():
             eval_func = inspect.getmembers(obj,
                     lambda x: inspect.ismethod(x) and x.__name__ == 'evaluate')
             try:
@@ -219,12 +219,17 @@ class Resources(Command):
         json.dump(function_dict, open(dest, 'wb'), indent=4)
 
     def clean(self):
-        for x in ('scripts', 'recipes', 'ebook-convert-complete'):
+        for x in ('scripts', 'ebook-convert-complete'):
             x = self.j(self.RESOURCES, x+'.pickle')
             if os.path.exists(x):
                 os.remove(x)
         from setup.commands import kakasi
         kakasi.clean()
+        for x in ('builtin_recipes.xml', 'builtin_recipes.zip',
+                'template-functions.json'):
+            x = self.j(self.RESOURCES, x)
+            if os.path.exists(x):
+                os.remove(x)
 
 
 
