@@ -102,6 +102,7 @@ class Metadata(object):
         @param other: None or a metadata object
         '''
         _data = copy.deepcopy(NULL_VALUES)
+        _data.pop('language')
         object.__setattr__(self, '_data', _data)
         if other is not None:
             self.smart_update(other)
@@ -136,6 +137,11 @@ class Metadata(object):
         _data = object.__getattribute__(self, '_data')
         if field in TOP_LEVEL_IDENTIFIERS:
             return _data.get('identifiers').get(field, None)
+        if field == 'language':
+            try:
+                return _data.get('languages', [])[0]
+            except:
+                return NULL_VALUES['language']
         if field in STANDARD_METADATA_FIELDS:
             return _data.get(field, None)
         try:
@@ -175,6 +181,11 @@ class Metadata(object):
             if not val:
                 val = copy.copy(NULL_VALUES.get('identifiers', None))
             self.set_identifiers(val)
+        elif field == 'language':
+            langs = []
+            if val and val.lower() != 'und':
+                langs = [val]
+            _data['languages'] = langs
         elif field in STANDARD_METADATA_FIELDS:
             if val is None:
                 val = copy.copy(NULL_VALUES.get(field, None))
@@ -529,8 +540,8 @@ class Metadata(object):
                             for t in st.intersection(ot):
                                 sidx = lstags.index(t)
                                 oidx = lotags.index(t)
-                                self_tags[sidx] = other.tags[oidx]
-                            self_tags += [t for t in other.tags if t.lower() in ot-st]
+                                self_tags[sidx] = other_tags[oidx]
+                            self_tags += [t for t in other_tags if t.lower() in ot-st]
                             setattr(self, x, self_tags)
 
             my_comments = getattr(self, 'comments', '')
@@ -553,9 +564,9 @@ class Metadata(object):
                 for attr in TOP_LEVEL_IDENTIFIERS:
                     copy_not_none(self, other, attr)
 
-        other_lang = getattr(other, 'language', None)
-        if other_lang and other_lang.lower() != 'und':
-            self.language = other_lang
+        other_lang = getattr(other, 'languages', [])
+        if other_lang and other_lang != ['und']:
+            self.languages = list(other_lang)
         if not getattr(self, 'series', None):
             self.series_index = None
 
@@ -706,8 +717,8 @@ class Metadata(object):
             fmt('Tags', u', '.join([unicode(t) for t in self.tags]))
         if self.series:
             fmt('Series', self.series + ' #%s'%self.format_series_index())
-        if not self.is_null('language'):
-            fmt('Language', self.language)
+        if not self.is_null('languages'):
+            fmt('Languages', ', '.join(self.languages))
         if self.rating is not None:
             fmt('Rating', self.rating)
         if self.timestamp is not None:
@@ -742,8 +753,8 @@ class Metadata(object):
         ans += [('ISBN', unicode(self.isbn))]
         ans += [(_('Tags'), u', '.join([unicode(t) for t in self.tags]))]
         if self.series:
-            ans += [(_('Series'), unicode(self.series)+ ' #%s'%self.format_series_index())]
-        ans += [(_('Language'), unicode(self.language))]
+            ans += [(_('Series'), unicode(self.series) + ' #%s'%self.format_series_index())]
+        ans += [(_('Languages'), u', '.join(self.languages))]
         if self.timestamp is not None:
             ans += [(_('Timestamp'), unicode(self.timestamp.isoformat(' ')))]
         if self.pubdate is not None:
