@@ -22,6 +22,7 @@ from calibre.ebooks.chardet import xml_to_unicode
 from calibre.ebooks.metadata.book.base import Metadata
 from calibre.library.comments import sanitize_comments_html
 from calibre.utils.date import parse_date
+from calibre.utils.localization import canonicalize_lang
 
 class Worker(Thread): # Get details {{{
 
@@ -73,6 +74,34 @@ class Worker(Thread): # Get details {{{
             9: ['sept'],
             12: ['déc'],
             },
+            'es': {
+                1: ['enero'],
+                2: ['febrero'],
+                3: ['marzo'],
+                4: ['abril'],
+                5: ['mayo'],
+                6: ['junio'],
+                7: ['julio'],
+                8: ['agosto'],
+                9: ['septiembre', 'setiembre'],
+                10: ['octubre'],
+                11: ['noviembre'],
+                12: ['diciembre'],
+            },
+                'jp': {
+            1: [u'1月'],
+            2: [u'2月'],
+            3: [u'3月'],
+            4: [u'4月'],
+            5: [u'5月'],
+            6: [u'6月'],
+            7: [u'7月'],
+            8: [u'8月'],
+            9: [u'9月'],
+            10: [u'10月'],
+            11: [u'11月'],
+            12: [u'12月'],
+            },
 
         }
 
@@ -85,13 +114,18 @@ class Worker(Thread): # Get details {{{
                  text()="Produktinformation" or \
                  text()="Dettagli prodotto" or \
                  text()="Product details" or \
-                 text()="Détails sur le produit"]/../div[@class="content"]
+                 text()="Détails sur le produit" or \
+                 text()="Detalles del producto" or \
+                 text()="登録情報"]/../div[@class="content"]
             '''
+        # Editor: is for Spanish
         self.publisher_xpath = '''
             descendant::*[starts-with(text(), "Publisher:") or \
                     starts-with(text(), "Verlag:") or \
                     starts-with(text(), "Editore:") or \
-                    starts-with(text(), "Editeur")]
+                    starts-with(text(), "Editeur") or \
+                    starts-with(text(), "Editor:") or \
+                    starts-with(text(), "出版社:")]
             '''
         self.language_xpath =    '''
             descendant::*[
@@ -99,17 +133,22 @@ class Worker(Thread): # Get details {{{
                 or text() = "Language" \
                 or text() = "Sprache:" \
                 or text() = "Lingua:" \
+                or text() = "Idioma:" \
                 or starts-with(text(), "Langue") \
+                or starts-with(text(), "言語") \
                 ]
             '''
+
         self.ratings_pat = re.compile(
-            r'([0-9.]+) (out of|von|su|étoiles sur) (\d+)( (stars|Sternen|stelle)){0,1}')
+            r'([0-9.]+) ?(out of|von|su|étoiles sur|つ星のうち|de un máximo de) ([\d\.]+)( (stars|Sternen|stelle|estrellas)){0,1}')
 
         lm = {
-                'en': ('English', 'Englisch'),
-                'fr': ('French', 'Français'),
-                'it': ('Italian', 'Italiano'),
-                'de': ('German', 'Deutsch'),
+                'eng': ('English', 'Englisch'),
+                'fra': ('French', 'Français'),
+                'ita': ('Italian', 'Italiano'),
+                'deu': ('German', 'Deutsch'),
+                'spa': ('Spanish', 'Espa\xf1ol', 'Espaniol'),
+                'jpn': ('Japanese', u'日本語'),
                 }
         self.lang_map = {}
         for code, names in lm.iteritems():
@@ -123,6 +162,7 @@ class Worker(Thread): # Get details {{{
         for i, vals in self.months.iteritems():
             for x in vals:
                 ans = ans.replace(x, self.english_months[i])
+        ans = ans.replace(' de ', ' ')
         return ans
 
     def run(self):
@@ -374,8 +414,11 @@ class Worker(Thread): # Get details {{{
     def parse_language(self, pd):
         for x in reversed(pd.xpath(self.language_xpath)):
             if x.tail:
-                ans = x.tail.strip()
-                ans = self.lang_map.get(ans, None)
+                raw = x.tail.strip()
+                ans = self.lang_map.get(raw, None)
+                if ans:
+                    return ans
+                ans = canonicalize_lang(ans)
                 if ans:
                     return ans
 # }}}
@@ -388,7 +431,7 @@ class Amazon(Source):
     capabilities = frozenset(['identify', 'cover'])
     touched_fields = frozenset(['title', 'authors', 'identifier:amazon',
         'identifier:isbn', 'rating', 'comments', 'publisher', 'pubdate',
-        'language'])
+        'languages'])
     has_html_comments = True
     supports_gzip_transfer_encoding = True
 
@@ -398,6 +441,8 @@ class Amazon(Source):
             'de' : _('Germany'),
             'uk' : _('UK'),
             'it' : _('Italy'),
+            'jp' : _('Japan'),
+            'es' : _('Spain'),
     }
 
     options = (
@@ -406,6 +451,22 @@ class Amazon(Source):
                     'country\'s Amazon website.'), choices=AMAZON_DOMAINS),
             )
 
+    def __init__(self, *args, **kwargs):
+        Source.__init__(self, *args, **kwargs)
+        self.set_amazon_id_touched_fields()
+
+    def save_settings(self, *args, **kwargs):
+        Source.save_settings(self, *args, **kwargs)
+        self.set_amazon_id_touched_fields()
+
+    def set_amazon_id_touched_fields(self):
+        ident_name = "identifier:amazon"
+        if self.domain != 'com':
+            ident_name += '_' + self.domain
+        tf = [x for x in self.touched_fields if not
+                x.startswith('identifier:amazon')] + [ident_name]
+        self.touched_fields = frozenset(tf)
+
     def get_domain_and_asin(self, identifiers):
         for key, val in identifiers.iteritems():
             key = key.lower()
@@ -483,13 +544,23 @@ class Amazon(Source):
             # Insufficient metadata to make an identify query
             return None, None
 
-        latin1q = dict([(x.encode('latin1', 'ignore'), y.encode('latin1',
+        # magic parameter to enable Japanese Shift_JIS encoding.
+        if domain == 'jp':
+            q['__mk_ja_JP'] = u'カタカナ'
+
+        if domain == 'jp':
+            encode_to = 'Shift_JIS'
+        else:
+            encode_to = 'latin1'
+        encoded_q = dict([(x.encode(encode_to, 'ignore'), y.encode(encode_to,
             'ignore')) for x, y in
             q.iteritems()])
         udomain = domain
         if domain == 'uk':
             udomain = 'co.uk'
-        url = 'http://www.amazon.%s/s/?'%udomain + urlencode(latin1q)
+        elif domain == 'jp':
+            udomain = 'co.jp'
+        url = 'http://www.amazon.%s/s/?'%udomain + urlencode(encoded_q)
         return url, domain
 
     # }}}
@@ -658,7 +729,7 @@ if __name__ == '__main__': # tests {{{
     # To run these test use: calibre-debug -e
     # src/calibre/ebooks/metadata/sources/amazon.py
     from calibre.ebooks.metadata.sources.test import (test_identify_plugin,
-            title_test, authors_test)
+            isbn_test, title_test, authors_test)
     com_tests = [ # {{{
 
             (  # Description has links
@@ -739,6 +810,31 @@ if __name__ == '__main__': # tests {{{
             ),
     ] # }}}
 
+    es_tests = [ # {{{
+            (
+                {'identifiers':{'isbn': '8483460831'}},
+                [title_test('Tiempos Interesantes',
+                    exact=True), authors_test(['Terry Pratchett'])
+                 ]
+
+            ),
+    ] # }}}
+
+    jp_tests = [ # {{{
+            ( # isbn -> title, authors
+                {'identifiers':{'isbn': '9784101302720' }},
+                [title_test(u'精霊の守り人',
+                    exact=True), authors_test([u'上橋 菜穂子'])
+                 ]
+            ),
+            ( # title, authors -> isbn (will use Shift_JIS encoding in query.)
+                {'title': u'考えない練習',
+                 'authors': [u'小池 龍之介']},
+                [isbn_test('9784093881067'), ]
+            ),
+    ] # }}}
+
     test_identify_plugin(Amazon.name, com_tests)
+    #test_identify_plugin(Amazon.name, es_tests)
 # }}}
 
