@@ -8,11 +8,11 @@ __docformat__ = 'restructuredtext en'
 import os
 from functools import partial
 
-from PyQt4.Qt import QModelIndex, QMenu
+from PyQt4.Qt import QModelIndex
 
 from calibre.gui2 import error_dialog, Dispatcher
 from calibre.gui2.tools import convert_single_ebook, convert_bulk_ebook
-from calibre.utils.config import prefs
+from calibre.utils.config import prefs, tweaks
 from calibre.gui2.actions import InterfaceAction
 from calibre.customize.ui import plugin_for_input_format
 
@@ -22,20 +22,22 @@ class ConvertAction(InterfaceAction):
     action_spec = (_('Convert books'), 'convert.png', None, _('C'))
     dont_add_to = frozenset(['menubar-device', 'toolbar-device', 'context-menu-device'])
     action_type = 'current'
+    action_add_menu = True
 
     def genesis(self):
-        cm = QMenu()
-        cm.addAction(_('Convert individually'), partial(self.convert_ebook,
+        m = self.convert_menu = self.qaction.menu()
+        cm = partial(self.create_menu_action, self.convert_menu)
+        cm('convert-individual', _('Convert individually'),
+                icon=self.qaction.icon(), triggered=partial(self.convert_ebook,
             False, bulk=False))
-        cm.addAction(_('Bulk convert'),
-                partial(self.convert_ebook, False, bulk=True))
-        cm.addSeparator()
-        ac = cm.addAction(
-                _('Create a catalog of the books in your calibre library'))
-        ac.triggered.connect(self.gui.iactions['Generate Catalog'].generate_catalog)
-        self.qaction.setMenu(cm)
+        cm('convert-bulk', _('Bulk convert'),
+                triggered=partial(self.convert_ebook, False, bulk=True))
+        m.addSeparator()
+        cm('create-catalog',
+                _('Create a catalog of the books in your calibre library'),
+                icon='catalog.png', shortcut=False,
+                triggered=self.gui.iactions['Generate Catalog'].generate_catalog)
         self.qaction.triggered.connect(self.convert_ebook)
-        self.convert_menu = cm
         self.conversion_jobs = {}
 
     def location_selected(self, loc):
@@ -118,6 +120,8 @@ class ConvertAction(InterfaceAction):
     def queue_convert_jobs(self, jobs, changed, bad, rows, previous,
             converted_func, extra_job_args=[]):
         for func, args, desc, fmt, id, temp_files in jobs:
+            func, _, same_fmt = func.partition(':')
+            same_fmt = same_fmt == 'same_fmt'
             input_file = args[0]
             input_fmt = os.path.splitext(input_file)[1]
             core_usage = 1
@@ -131,6 +135,7 @@ class ConvertAction(InterfaceAction):
                 job = self.gui.job_manager.run_job(Dispatcher(converted_func),
                                             func, args=args, description=desc,
                                             core_usage=core_usage)
+                job.conversion_of_same_fmt = same_fmt
                 args = [temp_files, fmt, id]+extra_job_args
                 self.conversion_jobs[job] = tuple(args)
 
@@ -166,14 +171,18 @@ class ConvertAction(InterfaceAction):
             if job.failed:
                 self.gui.job_exception(job)
                 return
+            same_fmt = getattr(job, 'conversion_of_same_fmt', False)
             fmtf = temp_files[-1].name
             if os.stat(fmtf).st_size < 1:
                 raise Exception(_('Empty output file, '
                     'probably the conversion process crashed'))
 
+            db = self.gui.current_db
+            if same_fmt and tweaks['save_original_format']:
+                db.save_original_format(book_id, fmt, notify=False)
+
             with open(temp_files[-1].name, 'rb') as data:
-                self.gui.library_view.model().db.add_format(book_id, \
-                    fmt, data, index_is_id=True)
+                db.add_format(book_id, fmt, data, index_is_id=True)
             self.gui.status_bar.show_message(job.description + \
                     (' completed'), 2000)
         finally:
