@@ -8,12 +8,12 @@ __docformat__ = 'restructuredtext en'
 import os
 from functools import partial
 
-from PyQt4.Qt import Qt, QMenu, QModelIndex, QTimer
+from PyQt4.Qt import QMenu, QModelIndex, QTimer
 
 from calibre.gui2 import error_dialog, Dispatcher, question_dialog
 from calibre.gui2.dialogs.metadata_bulk import MetadataBulkDialog
 from calibre.gui2.dialogs.confirm_delete import confirm
-from calibre.gui2.dialogs.tag_list_editor import TagListEditor
+from calibre.gui2.dialogs.device_category_editor import DeviceCategoryEditor
 from calibre.gui2.actions import InterfaceAction
 from calibre.ebooks.metadata import authors_to_string
 from calibre.utils.icu import sort_key
@@ -24,40 +24,41 @@ class EditMetadataAction(InterfaceAction):
     name = 'Edit Metadata'
     action_spec = (_('Edit metadata'), 'edit_input.png', None, _('E'))
     action_type = 'current'
+    action_add_menu = True
 
     def genesis(self):
-        self.create_action(spec=(_('Merge book records'), 'merge_books.png',
-            None, _('M')), attr='action_merge')
-        md = QMenu()
-        md.addAction(_('Edit metadata individually'),
-                partial(self.edit_metadata, False, bulk=False))
+        md = self.qaction.menu()
+        cm = partial(self.create_menu_action, md)
+        cm('individual', _('Edit metadata individually'), icon=self.qaction.icon(),
+                triggered=partial(self.edit_metadata, False, bulk=False))
         md.addSeparator()
-        md.addAction(_('Edit metadata in bulk'),
-                partial(self.edit_metadata, False, bulk=True))
+        cm('bulk', _('Edit metadata in bulk'),
+                triggered=partial(self.edit_metadata, False, bulk=True))
         md.addSeparator()
-        md.addAction(_('Download metadata and covers'), self.download_metadata,
-                Qt.ControlModifier+Qt.Key_D)
+        cm('download', _('Download metadata and covers'),
+                triggered=partial(self.download_metadata, ids=None),
+                shortcut='Ctrl+D')
         self.metadata_menu = md
 
         mb = QMenu()
-        mb.addAction(_('Merge into first selected book - delete others'),
-                self.merge_books)
+        cm2 = partial(self.create_menu_action, mb)
+        cm2('merge delete', _('Merge into first selected book - delete others'),
+                triggered=self.merge_books)
         mb.addSeparator()
-        mb.addAction(_('Merge into first selected book - keep others'),
-                partial(self.merge_books, safe_merge=True),
-                Qt.AltModifier+Qt.Key_M)
+        cm2('merge keep', _('Merge into first selected book - keep others'),
+                triggered=partial(self.merge_books, safe_merge=True),
+                shortcut='Alt+M')
         mb.addSeparator()
-        mb.addAction(_('Merge only formats into first selected book - delete others'),
-                partial(self.merge_books, merge_only_formats=True),
-                Qt.AltModifier+Qt.ShiftModifier+Qt.Key_M)
+        cm2('merge formats', _('Merge only formats into first selected book - delete others'),
+                triggered=partial(self.merge_books, merge_only_formats=True),
+                shortcut='Alt+Shift+M')
         self.merge_menu = mb
-        self.action_merge.setMenu(mb)
         md.addSeparator()
-        md.addAction(self.action_merge)
+        self.action_merge = cm('merge', _('Merge book records'), icon='merge_books.png',
+            shortcut=_('M'), triggered=self.merge_books)
+        self.action_merge.setMenu(mb)
 
         self.qaction.triggered.connect(self.edit_metadata)
-        self.qaction.setMenu(md)
-        self.action_merge.triggered.connect(self.merge_books)
 
     def location_selected(self, loc):
         enabled = loc == 'library'
@@ -284,7 +285,7 @@ class EditMetadataAction(InterfaceAction):
             if not confirm('<p>'+_(
                 'Book formats from the selected books will be merged '
                 'into the <b>first selected book</b> (%s). '
-                'Metadata in the first selected book will not be changed.'
+                'Metadata in the first selected book will not be changed. '
                 'Author, Title, ISBN and all other metadata will <i>not</i> be merged.<br><br>'
                 'After merger the second and subsequently '
                 'selected books, with any metadata they have will be <b>deleted</b>. <br><br>'
@@ -417,7 +418,7 @@ class EditMetadataAction(InterfaceAction):
                     db.set_custom(dest_id, dest_value, num=colnum)
               if db.field_metadata[key]['datatype'] in \
                 ('bool', 'int', 'float', 'rating', 'datetime') \
-                and not dest_value:
+                and dest_value is None:
                 db.set_custom(dest_id, src_value, num=colnum)
               if db.field_metadata[key]['datatype'] == 'series' \
                 and not dest_value:
@@ -441,14 +442,13 @@ class EditMetadataAction(InterfaceAction):
     def edit_device_collections(self, view, oncard=None):
         model = view.model()
         result = model.get_collections_with_ids()
-        d = TagListEditor(self.gui, tag_to_match=None, data=result, key=sort_key)
+        d = DeviceCategoryEditor(self.gui, tag_to_match=None, data=result, key=sort_key)
         d.exec_()
         if d.result() == d.Accepted:
             to_rename = d.to_rename # dict of new text to old ids
             to_delete = d.to_delete # list of ids
-            for text in to_rename:
-                for old_id in to_rename[text]:
-                    model.rename_collection(old_id, new_name=unicode(text))
+            for old_id, new_name in to_rename.iteritems():
+                model.rename_collection(old_id, new_name=unicode(new_name))
             for item in to_delete:
                 model.delete_collection_using_id(item)
             self.gui.upload_collections(model.db, view=view, oncard=oncard)
