@@ -229,14 +229,14 @@ class CheckLibraryDialog(QDialog):
         self.copy_button = QPushButton(_('Copy &to clipboard'))
         self.copy_button.setDefault(False)
         self.copy_button.clicked.connect(self.copy_to_clipboard)
-        self.ok_button = QPushButton('&Done')
+        self.ok_button = QPushButton(_('&Done'))
         self.ok_button.setDefault(True)
         self.ok_button.clicked.connect(self.accept)
-        self.delete_button = QPushButton('Delete &marked')
+        self.delete_button = QPushButton(_('Delete &marked'))
         self.delete_button.setToolTip(_('Delete marked files (checked subitems)'))
         self.delete_button.setDefault(False)
         self.delete_button.clicked.connect(self.delete_marked)
-        self.fix_button = QPushButton('&Fix marked')
+        self.fix_button = QPushButton(_('&Fix marked'))
         self.fix_button.setDefault(False)
         self.fix_button.setEnabled(False)
         self.fix_button.setToolTip(_('Fix marked sections (checked fixable items)'))
