@@ -16,13 +16,14 @@ from PyQt4.Qt import (QColor, Qt, QModelIndex, QSize, QApplication,
 
 from calibre.gui2 import UNDEFINED_QDATE, error_dialog
 from calibre.gui2.widgets import EnLineEdit
-from calibre.gui2.complete import MultiCompleteLineEdit
+from calibre.gui2.complete import MultiCompleteLineEdit, MultiCompleteComboBox
 from calibre.utils.date import now, format_date
 from calibre.utils.config import tweaks
 from calibre.utils.formatter import validation_formatter
 from calibre.utils.icu import sort_key
 from calibre.gui2.dialogs.comments_dialog import CommentsDialog
 from calibre.gui2.dialogs.template_dialog import TemplateDialog
+from calibre.gui2.languages import LanguagesEdit
 
 
 class RatingDelegate(QStyledItemDelegate): # {{{
@@ -155,7 +156,7 @@ class TextDelegate(QStyledItemDelegate): # {{{
     def __init__(self, parent):
         '''
         Delegate for text data. If auto_complete_function needs to return a list
-        of text items to auto-complete with. The funciton is None no
+        of text items to auto-complete with. If the function is None no
         auto-complete will be used.
         '''
         QStyledItemDelegate.__init__(self, parent)
@@ -166,13 +167,26 @@ class TextDelegate(QStyledItemDelegate): # {{{
 
     def createEditor(self, parent, option, index):
         if self.auto_complete_function:
-            editor = MultiCompleteLineEdit(parent)
+            editor = MultiCompleteComboBox(parent)
             editor.set_separator(None)
             complete_items = [i[1] for i in self.auto_complete_function()]
             editor.update_items_cache(complete_items)
+            for item in sorted(complete_items, key=sort_key):
+                editor.addItem(item)
+            ct = index.data(Qt.DisplayRole).toString()
+            editor.setEditText(ct)
+            editor.lineEdit().selectAll()
         else:
             editor = EnLineEdit(parent)
         return editor
+
+    def setModelData(self, editor, model, index):
+        if isinstance(editor, MultiCompleteComboBox):
+            val = editor.lineEdit().text()
+            model.setData(index, QVariant(val), Qt.EditRole)
+        else:
+            QStyledItemDelegate.setModelData(self, editor, model, index)
+
 #}}}
 
 class CompleteDelegate(QStyledItemDelegate): # {{{
@@ -188,7 +202,7 @@ class CompleteDelegate(QStyledItemDelegate): # {{{
     def createEditor(self, parent, option, index):
         if self.db and hasattr(self.db, self.items_func_name):
             col = index.model().column_map[index.column()]
-            editor = MultiCompleteLineEdit(parent)
+            editor = MultiCompleteComboBox(parent)
             editor.set_separator(self.sep)
             editor.set_space_before_sep(self.space_before_sep)
             if self.sep == '&':
@@ -199,9 +213,36 @@ class CompleteDelegate(QStyledItemDelegate): # {{{
                 all_items = list(self.db.all_custom(
                     label=self.db.field_metadata.key_to_label(col)))
             editor.update_items_cache(all_items)
+            for item in sorted(all_items, key=sort_key):
+                editor.addItem(item)
+            ct = index.data(Qt.DisplayRole).toString()
+            editor.setEditText(ct)
+            editor.lineEdit().selectAll()
         else:
             editor = EnLineEdit(parent)
         return editor
+
+    def setModelData(self, editor, model, index):
+        if isinstance(editor, MultiCompleteComboBox):
+            val = editor.lineEdit().text()
+            model.setData(index, QVariant(val), Qt.EditRole)
+        else:
+            QStyledItemDelegate.setModelData(self, editor, model, index)
+# }}}
+
+class LanguagesDelegate(QStyledItemDelegate): # {{{
+
+    def createEditor(self, parent, option, index):
+        editor = LanguagesEdit(parent=parent)
+        editor.init_langs(index.model().db)
+        ct = index.data(Qt.DisplayRole).toString()
+        editor.setEditText(ct)
+        editor.lineEdit().selectAll()
+        return editor
+
+    def setModelData(self, editor, model, index):
+        val = ','.join(editor.lang_codes)
+        model.setData(index, QVariant(val), Qt.EditRole)
 # }}}
 
 class CcDateDelegate(QStyledItemDelegate): # {{{
@@ -275,16 +316,22 @@ class CcNumberDelegate(QStyledItemDelegate): # {{{
         col = m.column_map[index.column()]
         if m.custom_columns[col]['datatype'] == 'int':
             editor = QSpinBox(parent)
-            editor.setRange(-100, 100000000)
+            editor.setRange(-1000000, 100000000)
             editor.setSpecialValueText(_('Undefined'))
             editor.setSingleStep(1)
         else:
             editor = QDoubleSpinBox(parent)
             editor.setSpecialValueText(_('Undefined'))
-            editor.setRange(-100., 100000000)
+            editor.setRange(-1000000., 100000000)
             editor.setDecimals(2)
         return editor
 
+    def setModelData(self, editor, model, index):
+        val = editor.value()
+        if val == editor.minimum():
+            val = None
+        model.setData(index, QVariant(val), Qt.EditRole)
+
     def setEditorData(self, editor, index):
         m = index.model()
         val = m.db.data[index.row()][m.custom_columns[m.column_map[index.column()]]['rec_index']]
