@@ -4,16 +4,15 @@ __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 import sys, os, time, socket, traceback
 from functools import partial
 
-from PyQt4.Qt import QCoreApplication, QIcon, QObject, QTimer, \
-        QThread, pyqtSignal, Qt, QProgressDialog, QString, QPixmap, \
-        QSplashScreen, QApplication
+from PyQt4.Qt import (QCoreApplication, QIcon, QObject, QTimer,
+        QPixmap, QSplashScreen, QApplication)
 
-from calibre import prints, plugins
-from calibre.constants import iswindows, __appname__, isosx, DEBUG, \
-        filesystem_encoding
+from calibre import prints, plugins, force_unicode
+from calibre.constants import (iswindows, __appname__, isosx, DEBUG,
+        filesystem_encoding)
 from calibre.utils.ipc import ADDRESS, RC
-from calibre.gui2 import ORG_NAME, APP_UID, initialize_file_icon_provider, \
-    Application, choose_dir, error_dialog, question_dialog, gprefs
+from calibre.gui2 import (ORG_NAME, APP_UID, initialize_file_icon_provider,
+    Application, choose_dir, error_dialog, question_dialog, gprefs)
 from calibre.gui2.main_window import option_parser as _option_parser
 from calibre.utils.config import prefs, dynamic
 from calibre.library.database2 import LibraryDatabase2
@@ -110,36 +109,9 @@ def get_library_path(parent=None):
                 default_dir=get_default_library_path())
     return library_path
 
-class DBRepair(QThread):
-
-    repair_done = pyqtSignal(object, object)
-    progress = pyqtSignal(object, object)
-
-    def __init__(self, library_path, parent, pd):
-        QThread.__init__(self, parent)
-        self.library_path = library_path
-        self.pd = pd
-        self.progress.connect(self._callback, type=Qt.QueuedConnection)
-
-    def _callback(self, num, is_length):
-        if is_length:
-            self.pd.setRange(0, num-1)
-            num = 0
-        self.pd.setValue(num)
-
-    def callback(self, num, is_length):
-        self.progress.emit(num, is_length)
-
-    def run(self):
-        from calibre.debug import reinit_db
-        try:
-            reinit_db(os.path.join(self.library_path, 'metadata.db'),
-                    self.callback)
-            db = LibraryDatabase2(self.library_path)
-            tb = None
-        except:
-            db, tb = None, traceback.format_exc()
-        self.repair_done.emit(db, tb)
+def repair_library(library_path):
+    from calibre.gui2.dialogs.restore_library import repair_library_at
+    return repair_library_at(library_path)
 
 class GuiRunner(QObject):
     '''Make sure an event loop is running before starting the main work of
@@ -170,7 +142,11 @@ class GuiRunner(QObject):
         sys.excepthook = main.unhandled_exception
         if len(self.args) > 1:
             p = os.path.abspath(self.args[1])
-            add_filesystem_book(p)
+            if os.path.isdir(p):
+                prints('Ignoring directory passed as command line argument:',
+                        self.args[1])
+            else:
+                add_filesystem_book(p)
         self.app.file_event_hook = add_filesystem_book
         self.main = main
 
@@ -180,9 +156,6 @@ class GuiRunner(QObject):
         raise SystemExit(1)
 
     def initialize_db_stage2(self, db, tb):
-        repair_pd = getattr(self, 'repair_pd', None)
-        if repair_pd is not None:
-            repair_pd.cancel()
 
         if db is None and tb is not None:
             # DB Repair failed
@@ -215,23 +188,16 @@ class GuiRunner(QObject):
             db = LibraryDatabase2(self.library_path)
         except (sqlite.Error, DatabaseException):
             repair = question_dialog(self.splash_screen, _('Corrupted database'),
-                    _('Your calibre database appears to be corrupted. Do '
-                    'you want calibre to try and repair it automatically? '
-                    'If you say No, a new empty calibre library will be created.'),
+                    _('The library database at %s appears to be corrupted. Do '
+                    'you want calibre to try and rebuild it automatically? '
+                    'The rebuild may not be completely successful. '
+                    'If you say No, a new empty calibre library will be created.')
+                    % force_unicode(self.library_path, filesystem_encoding),
                     det_msg=traceback.format_exc()
                     )
             if repair:
-                self.repair_pd = QProgressDialog(_('Repairing database. This '
-                    'can take a very long time for a large collection'), QString(),
-                    0, 0)
-                self.repair_pd.setWindowModality(Qt.WindowModal)
-                self.repair_pd.show()
-
-                self.repair = DBRepair(self.library_path, self, self.repair_pd)
-                self.repair.repair_done.connect(self.initialize_db_stage2,
-                        type=Qt.QueuedConnection)
-                self.repair.start()
-                return
+                if repair_library(self.library_path):
+                    db = LibraryDatabase2(self.library_path)
         except:
             error_dialog(self.splash_screen, _('Bad database location'),
                     _('Bad database location %r. Will start with '
