@@ -254,6 +254,10 @@ class ResultsView(QTableView): # {{{
             '<h2>%s</h2>'%book.title,
             '<div><i>%s</i></div>'%authors_to_string(book.authors),
         ]
+        if not book.is_null('series'):
+            series = book.format_field('series')
+            if series[1]:
+                parts.append('<div>%s: %s</div>'%series)
         if not book.is_null('rating'):
             parts.append('<div>%s</div>'%('\u2605'*int(book.rating)))
         parts.append('</center>')
@@ -722,8 +726,8 @@ class CoversWidget(QWidget): # {{{
         if num < 2:
             txt = _('Could not find any covers for <b>%s</b>')%self.book.title
         else:
-            txt = _('Found <b>%d</b> covers of %s. Pick the one you like'
-                    ' best.')%(num-1, self.title)
+            txt = _('Found <b>%(num)d</b> covers of %(title)s. Pick the one you like'
+                    ' best.')%dict(num=num-1, title=self.title)
         self.msg.setText(txt)
 
         self.finished.emit()
