@@ -99,6 +99,8 @@ typedef unsigned short QRgb565;
 #define PFREAL_ONE (1 << PFREAL_SHIFT)
 #define PFREAL_HALF (PFREAL_ONE >> 1)
 
+#define TEXT_FLAGS (Qt::TextWordWrap|Qt::TextWrapAnywhere|Qt::TextHideMnemonic|Qt::AlignCenter)
+
 inline PFreal fmul(PFreal a, PFreal b)
 {
   return ((long long)(a))*((long long)(b)) >> PFREAL_SHIFT;
@@ -401,6 +403,7 @@ private:
   QImage* surface(int slideIndex);
   void triggerRender();
   void resetSlides();
+  void render_text(QPainter*, int);
 };
 
 PictureFlowPrivate::PictureFlowPrivate(PictureFlow* w, int queueLength_)
@@ -663,6 +666,34 @@ void PictureFlowPrivate::triggerRender()
   triggerTimer.start();
 }
 
+void PictureFlowPrivate::render_text(QPainter *painter, int index) {
+    QRect brect, brect2;
+    int buffer_width, buffer_height;
+    QString caption, subtitle;
+
+    caption = slideImages->caption(index);
+    subtitle = slideImages->subtitle(index);
+    buffer_width = buffer.width(); buffer_height = buffer.height();
+
+    brect = painter->boundingRect(QRect(0, 0, buffer_width, fontSize), TEXT_FLAGS, caption);
+    brect2 = painter->boundingRect(QRect(0, 0, buffer_width, fontSize), TEXT_FLAGS, subtitle);
+
+    // So that if there is no subtitle, the caption is not flush with the bottom
+    if (brect2.height() < fontSize) brect2.setHeight(fontSize);
+
+    // So that the text does not occupy more than the lower half of the buffer
+    if (brect.height() > ((int)(buffer.height()/3.0)) - fontSize*2)
+        brect.setHeight(((int)buffer.height()/3.0) - fontSize*2);
+
+    brect.moveTop(buffer_height - (brect.height() + brect2.height()));
+    //printf("top: %d, height: %d\n", brect.top(), brect.height());
+    //
+    painter->drawText(brect, TEXT_FLAGS, caption);
+    
+    brect2.moveTop(buffer_height - brect2.height());
+    painter->drawText(brect2, TEXT_FLAGS, slideImages->subtitle(index));
+}
+
 // Render the slides. Updates only the offscreen buffer.
 void PictureFlowPrivate::render()
 {
@@ -708,10 +739,7 @@ void PictureFlowPrivate::render()
     //painter.setPen(QColor(255,255,255,127));
 
     if (centerIndex < slideCount() && centerIndex > -1) { 
-    	painter.drawText( QRect(0,0, buffer.width(), buffer.height()*2-fontSize*4),
-                      Qt::AlignCenter, slideImages->caption(centerIndex));
-    	painter.drawText( QRect(0,0, buffer.width(), buffer.height()*2-fontSize*2),
-                      Qt::AlignCenter, slideImages->subtitle(centerIndex));
+        render_text(&painter, centerIndex);
     }
 
     painter.end();
@@ -764,20 +792,12 @@ void PictureFlowPrivate::render()
 
     painter.setPen(QColor(255,255,255, (255-fade) ));
     if (leftTextIndex < sc && leftTextIndex > -1) {
-    	painter.drawText( QRect(0,0, buffer.width(), buffer.height()*2 - fontSize*4),
-                      Qt::AlignCenter, slideImages->caption(leftTextIndex));
-    	painter.drawText( QRect(0,0, buffer.width(), buffer.height()*2 - fontSize*2),
-                      Qt::AlignCenter, slideImages->subtitle(leftTextIndex));
-
+        render_text(&painter, leftTextIndex);
     }
 
     painter.setPen(QColor(255,255,255, fade));
     if (leftTextIndex+1 < sc && leftTextIndex > -2) {
-    	painter.drawText( QRect(0,0, buffer.width(), buffer.height()*2 - fontSize*4),
-                      Qt::AlignCenter, slideImages->caption(leftTextIndex+1));
-    	painter.drawText( QRect(0,0, buffer.width(), buffer.height()*2 - fontSize*2),
-                      Qt::AlignCenter, slideImages->subtitle(leftTextIndex+1));
-
+        render_text(&painter, leftTextIndex+1);
     }
 
     painter.end();
@@ -1332,6 +1352,7 @@ void PictureFlow::mousePressEvent(QMouseEvent* event)
 
 void PictureFlow::mouseReleaseEvent(QMouseEvent* event)
 {
+  bool accepted = false;
   int sideWidth = (d->buffer.width() - slideSize().width()) /2;
 
   if (d->singlePress)
@@ -1339,13 +1360,20 @@ void PictureFlow::mouseReleaseEvent(QMouseEvent* event)
     if (event->x() < sideWidth )
     {
       showPrevious();
+      accepted = true;
     } else if ( event->x() > sideWidth + slideSize().width() ) {
       showNext();
+      accepted = true;
     } else {
-      emit itemActivated(d->getTarget());
+        if (event->button() == Qt::LeftButton) {
+              emit itemActivated(d->getTarget());
+              accepted = true;
+        }
     }
 
-    event->accept();
+    if (accepted) {
+        event->accept();
+    }
   }
 
   emit inputReceived();
