@@ -58,7 +58,9 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         self.device_to_formats_map = {}
         for device in device_plugins():
             n = device_name_for_plugboards(device)
-            self.device_to_formats_map[n] = device.FORMATS
+            self.device_to_formats_map[n] = set(device.FORMATS)
+            if getattr(device, 'CAN_DO_DEVICE_DB_PLUGBOARD', False):
+                self.device_to_formats_map[n].add('device_db')
             if n not in self.devices:
                 self.devices.append(n)
         self.devices.sort(cmp=lambda x, y: cmp(x.lower(), y.lower()))
@@ -358,5 +360,5 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
 if __name__ == '__main__':
     from PyQt4.Qt import QApplication
     app = QApplication([])
-    test_widget('Import/Export', 'plugboards')
+    test_widget('Import/Export', 'Plugboard')
 
