@@ -16,10 +16,10 @@ from collections import OrderedDict
 
 from PyQt4.Qt import (Qt, SIGNAL, QTimer, QHelpEvent, QAction,
                      QMenu, QIcon, pyqtSignal, QUrl,
-                     QDialog, QSystemTrayIcon, QApplication, QKeySequence)
+                     QDialog, QSystemTrayIcon, QApplication)
 
-from calibre import  prints
-from calibre.constants import __appname__, isosx
+from calibre import prints, force_unicode
+from calibre.constants import __appname__, isosx, filesystem_encoding
 from calibre.utils.config import prefs, dynamic
 from calibre.utils.ipc.server import Server
 from calibre.library.database2 import LibraryDatabase2
@@ -40,7 +40,8 @@ from calibre.gui2.init import LibraryViewMixin, LayoutMixin
 from calibre.gui2.search_box import SearchBoxMixin, SavedSearchBoxMixin
 from calibre.gui2.search_restriction_mixin import SearchRestrictionMixin
 from calibre.gui2.tag_browser.ui import TagBrowserMixin
-
+from calibre.gui2.keyboard import Manager
+from calibre.library.sqlite import sqlite, DatabaseException
 
 class Listener(Thread): # {{{
 
@@ -104,6 +105,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin, # {{{
     def __init__(self, opts, parent=None, gui_debug=None):
         global _gui
         MainWindow.__init__(self, opts, parent=parent, disable_automatic_gc=True)
+        self.keyboard = Manager(self)
         _gui = self
         self.opts = opts
         self.device_connected = None
@@ -238,7 +240,8 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin, # {{{
         self.eject_action.setEnabled(False)
         self.addAction(self.quit_action)
         self.system_tray_menu.addAction(self.quit_action)
-        self.quit_action.setShortcut(QKeySequence(Qt.CTRL + Qt.Key_Q))
+        self.keyboard.register_shortcut('quit calibre', _('Quit calibre'),
+                default_keys=('Ctrl+Q',), action=self.quit_action)
         self.system_tray_icon.setContextMenu(self.system_tray_menu)
         self.connect(self.quit_action, SIGNAL('triggered(bool)'), self.quit)
         self.connect(self.donate_action, SIGNAL('triggered(bool)'), self.donate)
@@ -249,7 +252,9 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin, # {{{
 
         self.esc_action = QAction(self)
         self.addAction(self.esc_action)
-        self.esc_action.setShortcut(QKeySequence(Qt.Key_Escape))
+        self.keyboard.register_shortcut('clear current search',
+                _('Clear the current search'), default_keys=('Esc',),
+                action=self.esc_action)
         self.esc_action.triggered.connect(self.esc)
 
         ####################### Start spare job server ########################
@@ -308,6 +313,8 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin, # {{{
                 self.height())
         self.resize(self.width(), self._calculated_available_height)
 
+        self.build_context_menus()
+
         for ac in self.iactions.values():
             try:
                 ac.gui_layout_complete()
@@ -338,6 +345,8 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin, # {{{
                     raise
         self.device_manager.set_current_library_uuid(db.library_id)
 
+        self.keyboard.finalize()
+
         # Collect cycles now
         gc.collect()
 
@@ -466,7 +475,8 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin, # {{{
     def booklists(self):
         return self.memory_view.model().db, self.card_a_view.model().db, self.card_b_view.model().db
 
-    def library_moved(self, newloc, copy_structure=False, call_close=True):
+    def library_moved(self, newloc, copy_structure=False, call_close=True,
+            allow_rebuild=False):
         if newloc is None: return
         default_prefs = None
         try:
@@ -475,7 +485,26 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin, # {{{
                 default_prefs = olddb.prefs
         except:
             olddb = None
-        db = LibraryDatabase2(newloc, default_prefs=default_prefs)
+        try:
+            db = LibraryDatabase2(newloc, default_prefs=default_prefs)
+        except (DatabaseException, sqlite.Error):
+            if not allow_rebuild: raise
+            import traceback
+            repair = question_dialog(self, _('Corrupted database'),
+                    _('The library database at %s appears to be corrupted. Do '
+                    'you want calibre to try and rebuild it automatically? '
+                    'The rebuild may not be completely successful.')
+                    % force_unicode(newloc, filesystem_encoding),
+                    det_msg=traceback.format_exc()
+                    )
+            if repair:
+                from calibre.gui2.dialogs.restore_library import repair_library_at
+                if repair_library_at(newloc, parent=self):
+                    db = LibraryDatabase2(newloc, default_prefs=default_prefs)
+                else:
+                    return
+            else:
+                return
         if self.content_server is not None:
             self.content_server.set_database(db)
         self.library_path = newloc
@@ -513,6 +542,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin, # {{{
             self.card_a_view.reset()
             self.card_b_view.reset()
         self.device_manager.set_current_library_uuid(db.library_id)
+        self.library_view.set_current_row(0)
         # Run a garbage collection now so that it does not freeze the
         # interface later
         gc.collect()
