@@ -121,7 +121,8 @@ def isoformat(date_time, assume_utc=False, as_utc=True, sep='T'):
         date_time = date_time.replace(tzinfo=_utc_tz if assume_utc else
                 _local_tz)
     date_time = date_time.astimezone(_utc_tz if as_utc else _local_tz)
-    return unicode(date_time.isoformat(sep))
+    # str(sep) because isoformat barfs with unicode sep on python 2.x
+    return unicode(date_time.isoformat(str(sep)))
 
 def as_local_time(date_time, assume_utc=True):
     if not hasattr(date_time, 'tzinfo'):
@@ -131,6 +132,14 @@ def as_local_time(date_time, assume_utc=True):
                 _local_tz)
     return date_time.astimezone(_local_tz)
 
+def as_utc(date_time, assume_utc=True):
+    if not hasattr(date_time, 'tzinfo'):
+        return date_time
+    if date_time.tzinfo is None:
+        date_time = date_time.replace(tzinfo=_utc_tz if assume_utc else
+                _local_tz)
+    return date_time.astimezone(_utc_tz)
+
 def now():
     return datetime.now().replace(tzinfo=_local_tz)
 
@@ -161,32 +170,40 @@ def format_date(dt, format, assume_utc=False, as_utc=False):
     if format == 'iso':
         return isoformat(dt, assume_utc=assume_utc, as_utc=as_utc)
 
+    if dt == UNDEFINED_DATE:
+        return ''
+
     strf = partial(strftime, t=dt.timetuple())
 
-    def format_day(mo):
-        l = len(mo.group(0))
+    def format_day(dy):
+        l = len(dy)
         if l == 1: return '%d'%dt.day
         if l == 2: return '%02d'%dt.day
         if l == 3: return strf('%a')
         return strf('%A')
 
     def format_month(mo):
-        l = len(mo.group(0))
+        l = len(mo)
         if l == 1: return '%d'%dt.month
         if l == 2: return '%02d'%dt.month
         if l == 3: return strf('%b')
         return strf('%B')
 
-    def format_year(mo):
-        if len(mo.group(0)) == 2: return '%02d'%(dt.year % 100)
+    def format_year(yr):
+        if len(yr) == 2: return '%02d'%(dt.year % 100)
         return '%04d'%dt.year
 
-    if dt == UNDEFINED_DATE:
-        return ''
+    def repl_func(mo):
+        s = mo.group(0)
+        if s is None:
+            return ''
+        if s[0] == 'd':
+            return format_day(s)
+        if s[0] == 'M':
+            return format_month(s)
+        return format_year(s)
 
-    format = re.sub('d{1,4}', format_day, format)
-    format = re.sub('M{1,4}', format_month, format)
-    return re.sub('yyyy|yy', format_year, format)
+    return re.sub('(d{1,4}|M{1,4}|(?:yyyy|yy))', repl_func, format)
 
 def replace_months(datestr, clang):
     # Replace months by english equivalent for parse_date
