@@ -10,55 +10,70 @@ __docformat__ = 'restructuredtext en'
 
 import inspect, re, traceback
 
+from calibre import human_readable
 from calibre.utils.titlecase import titlecase
 from calibre.utils.icu import capitalize, strcmp, sort_key
 from calibre.utils.date import parse_date, format_date, now, UNDEFINED_DATE
-
+from calibre.utils.localization import calibre_langcode_to_name, canonicalize_lang
 
 class FormatterFunctions(object):
 
     def __init__(self):
-        self.builtins = {}
-        self.functions = {}
+        self._builtins = {}
+        self._functions = {}
 
     def register_builtin(self, func_class):
         if not isinstance(func_class, FormatterFunction):
             raise ValueError('Class %s is not an instance of FormatterFunction'%(
                                     func_class.__class__.__name__))
         name = func_class.name
-        if name in self.functions:
+        if name in self._functions:
             raise ValueError('Name %s already used'%name)
-        self.builtins[name] = func_class
-        self.functions[name] = func_class
+        self._builtins[name] = func_class
+        self._functions[name] = func_class
+        for a in func_class.aliases:
+            self._functions[a] = func_class
 
     def register_function(self, func_class):
         if not isinstance(func_class, FormatterFunction):
             raise ValueError('Class %s is not an instance of FormatterFunction'%(
                                     func_class.__class__.__name__))
         name = func_class.name
-        if name in self.functions:
+        if name in self._functions:
             raise ValueError('Name %s already used'%name)
-        self.functions[name] = func_class
+        self._functions[name] = func_class
 
     def get_builtins(self):
-        return self.builtins
+        return self._builtins
+
+    def get_builtins_and_aliases(self):
+        res = {}
+        for f in self._builtins.itervalues():
+            res[f.name] = f
+            for a in f.aliases:
+                res[a] = f
+        return res
 
     def get_functions(self):
-        return self.functions
+        return self._functions
 
     def reset_to_builtins(self):
-        self.functions = dict([t for t in self.builtins.items()])
+        self._functions = {}
+        for n,c in self._builtins.items():
+            self._functions[n] = c
+            for a in c.aliases:
+                self._functions[a] = c
 
 formatter_functions = FormatterFunctions()
 
 
-
 class FormatterFunction(object):
 
     doc = _('No documentation provided')
     name = 'no name provided'
     category = 'Unknown'
     arg_count = 0
+    aliases = []
 
     def evaluate(self, formatter, kwargs, mi, locals, *args):
         raise NotImplementedError()
@@ -72,7 +87,6 @@ class FormatterFunction(object):
         if isinstance(ret, list):
             return ','.join(list)
 
-all_builtin_functions = []
 class BuiltinFormatterFunction(FormatterFunction):
     def __init__(self):
         formatter_functions.register_builtin(self)
@@ -83,7 +97,6 @@ class BuiltinFormatterFunction(FormatterFunction):
         except:
             lines = []
         self.program_text = ''.join(lines)
-        all_builtin_functions.append(self)
 
 class BuiltinStrcmp(BuiltinFormatterFunction):
     name = 'strcmp'
@@ -131,6 +144,19 @@ class BuiltinStrcat(BuiltinFormatterFunction):
             res += args[i]
         return res
 
+class BuiltinStrlen(BuiltinFormatterFunction):
+    name = 'strlen'
+    arg_count = 1
+    category = 'String Manipulation'
+    __doc__ = doc = _('strlen(a) -- Returns the length of the string passed as '
+            'the argument')
+
+    def evaluate(self, formatter, kwargs, mi, locals, a):
+        try:
+            return len(a)
+        except:
+            return -1
+
 class BuiltinAdd(BuiltinFormatterFunction):
     name = 'add'
     arg_count = 2
@@ -340,6 +366,40 @@ class BuiltinSwitch(BuiltinFormatterFunction):
                 return args[i+1]
             i += 2
 
+class BuiltinStrcatMax(BuiltinFormatterFunction):
+    name = 'strcat_max'
+    arg_count = -1
+    category = 'String Manipulation'
+    __doc__ = doc = _('strcat_max(max, string1, prefix2, string2, ...) -- '
+            'Returns a string formed by concatenating the arguments. The '
+            'returned value is initialized to string1. `Prefix, string` '
+            'pairs are added to the end of the value as long as the '
+            'resulting string length is less than `max`. String1 is returned '
+            'even if string1 is longer than max. You can pass as many '
+            '`prefix, string` pairs as you wish.')
+
+    def evaluate(self, formatter, kwargs, mi, locals, *args):
+        if len(args) < 2:
+            raise ValueError(_('strcat_max requires 2 or more arguments'))
+        if (len(args) % 2) != 0:
+            raise ValueError(_('strcat_max requires an even number of arguments'))
+        try:
+            max = int(args[0])
+        except:
+            raise ValueError(_('first argument to strcat_max must be an integer'))
+
+        i = 2
+        result = args[1]
+        try:
+            while i < len(args):
+                if (len(result) + len(args[i]) + len(args[i+1])) > max:
+                    break
+                result = result + args[i] + args[i+1]
+                i += 2
+        except:
+            pass
+        return result.strip()
+
 class BuiltinInList(BuiltinFormatterFunction):
     name = 'in_list'
     arg_count = 5
@@ -427,7 +487,7 @@ class BuiltinSwapAroundComma(BuiltinFormatterFunction):
             'returns val unchanged')
 
     def evaluate(self, formatter, kwargs, mi, locals, val):
-        return re.sub(r'^(.*?),(.*$)', r'\2 \1', val, flags=re.I)
+        return re.sub(r'^(.*?),\s*(.*$)', r'\2 \1', val, flags=re.I).strip()
 
 class BuiltinIfempty(BuiltinFormatterFunction):
     name = 'ifempty'
@@ -497,7 +557,7 @@ class BuiltinListitem(BuiltinFormatterFunction):
         index = int(index)
         val = val.split(sep)
         try:
-            return val[index]
+            return val[index].strip()
         except:
             return ''
 
@@ -506,7 +566,7 @@ class BuiltinSelect(BuiltinFormatterFunction):
     arg_count = 2
     category = 'List Lookup'
     __doc__ = doc = _('select(val, key) -- interpret the value as a comma-separated list '
-            'of items, with the items being "id:value". Find the pair with the'
+            'of items, with the items being "id:value". Find the pair with the '
             'id equal to key, and return the corresponding value.'
             )
 
@@ -519,6 +579,80 @@ class BuiltinSelect(BuiltinFormatterFunction):
                 return v[len(key)+1:]
         return ''
 
+class BuiltinFormatsModtimes(BuiltinFormatterFunction):
+    name = 'formats_modtimes'
+    arg_count = 1
+    category = 'Get values from metadata'
+    __doc__ = doc = _('formats_modtimes(date_format) -- return a comma-separated '
+                  'list of colon_separated items representing modification times '
+                  'for the formats of a book. The date_format parameter '
+                  'specifies how the date is to be formatted. See the '
+                  'date_format function for details. You can use the select '
+                  'function to get the mod time for a specific '
+                  'format. Note that format names are always uppercase, '
+                  'as in EPUB.'
+            )
+
+    def evaluate(self, formatter, kwargs, mi, locals, fmt):
+        fmt_data = mi.get('format_metadata', {})
+        return ','.join(k.upper()+':'+format_date(v['mtime'], fmt)
+                        for k,v in fmt_data.iteritems())
+
+class BuiltinFormatsSizes(BuiltinFormatterFunction):
+    name = 'formats_sizes'
+    arg_count = 0
+    category = 'Get values from metadata'
+    __doc__ = doc = _('formats_sizes() -- return a comma-separated list of '
+                      'colon_separated items representing sizes in bytes '
+                      'of the formats of a book. You can use the select '
+                      'function to get the size for a specific '
+                      'format. Note that format names are always uppercase, '
+                      'as in EPUB.'
+            )
+
+    def evaluate(self, formatter, kwargs, mi, locals):
+        fmt_data = mi.get('format_metadata', {})
+        return ','.join(k.upper()+':'+str(v['size']) for k,v in fmt_data.iteritems())
+
+class BuiltinHumanReadable(BuiltinFormatterFunction):
+    name = 'human_readable'
+    arg_count = 1
+    category = 'Formatting values'
+    __doc__ = doc = _('human_readable(v) -- return a string '
+                      'representing the number v in KB, MB, GB, etc.'
+            )
+
+    def evaluate(self, formatter, kwargs, mi, locals, val):
+        try:
+            return human_readable(round(float(val)))
+        except:
+            return ''
+
+class BuiltinFormatNumber(BuiltinFormatterFunction):
+    name = 'format_number'
+    arg_count = 2
+    category = 'Formatting values'
+    __doc__ = doc = _('format_number(v, template) -- format the number v using '
+                  'a python formatting template such as "{0:5.2f}" or '
+                  '"{0:,d}" or "${0:5,.2f}". The field_name part of the '
+                  'template must be a 0 (zero) (the "{0:" in the above examples). '
+                  'See the template language and python documentation for more '
+                  'examples. Returns the empty string if formatting fails.'
+            )
+
+    def evaluate(self, formatter, kwargs, mi, locals, val, template):
+        if val == '' or val == 'None':
+            return ''
+        try:
+            return template.format(float(val))
+        except:
+            pass
+        try:
+            return template.format(int(val))
+        except:
+            pass
+        return ''
+
 class BuiltinSublist(BuiltinFormatterFunction):
     name = 'sublist'
     arg_count = 4
@@ -541,7 +675,8 @@ class BuiltinSublist(BuiltinFormatterFunction):
             return ''
         si = int(start_index)
         ei = int(end_index)
-        val = val.split(sep)
+        # allow empty list items so counts are what the user expects
+        val = [v.strip() for v in val.split(sep)]
         try:
             if ei == 0:
                 return sep.join(val[si:])
@@ -591,7 +726,7 @@ class BuiltinSubitems(BuiltinFormatterFunction):
 class BuiltinFormatDate(BuiltinFormatterFunction):
     name = 'format_date'
     arg_count = 2
-    category = 'Date functions'
+    category = 'Formatting values'
     __doc__ = doc = _('format_date(val, format_string) -- format the value, '
             'which must be a date, using the format_string, returning a string. '
             'The formatting codes are: '
@@ -679,6 +814,18 @@ class BuiltinOndevice(BuiltinFormatterFunction):
             return _('Yes')
         return ''
 
+class BuiltinHasCover(BuiltinFormatterFunction):
+    name = 'has_cover'
+    arg_count = 0
+    category = 'Get values from metadata'
+    __doc__ = doc = _('has_cover() -- return Yes if the book has a cover, '
+                      'otherwise return the empty string')
+
+    def evaluate(self, formatter, kwargs, mi, locals):
+        if mi.has_cover:
+            return _('Yes')
+        return ''
+
 class BuiltinFirstNonEmpty(BuiltinFormatterFunction):
     name = 'first_non_empty'
     arg_count = -1
@@ -742,16 +889,17 @@ class BuiltinNot(BuiltinFormatterFunction):
     def evaluate(self, formatter, kwargs, mi, locals, val):
         return '' if val else '1'
 
-class BuiltinMergeLists(BuiltinFormatterFunction):
-    name = 'merge_lists'
+class BuiltinListUnion(BuiltinFormatterFunction):
+    name = 'list_union'
     arg_count = 3
     category = 'List Manipulation'
-    __doc__ = doc = _('merge_lists(list1, list2, separator) -- '
+    __doc__ = doc = _('list_union(list1, list2, separator) -- '
             'return a list made by merging the items in list1 and list2, '
             'removing duplicate items using a case-insensitive compare. If '
             'items differ in case, the one in list1 is used. '
             'The items in list1 and list2 are separated by separator, as are '
             'the items in the returned list.')
+    aliases = ['merge_lists']
 
     def evaluate(self, formatter, kwargs, mi, locals, list1, list2, separator):
         l1 = [l.strip() for l in list1.split(separator) if l.strip()]
@@ -764,7 +912,76 @@ class BuiltinMergeLists(BuiltinFormatterFunction):
         for i in l2:
             if icu_lower(i) not in lcl1:
                 res.append(i)
-        return ', '.join(sorted(res, key=sort_key))
+        return ', '.join(res)
+
+class BuiltinListDifference(BuiltinFormatterFunction):
+    name = 'list_difference'
+    arg_count = 3
+    category = 'List Manipulation'
+    __doc__ = doc = _('list_difference(list1, list2, separator) -- '
+            'return a list made by removing from list1 any item found in list2, '
+            'using a case-insensitive compare. The items in list1 and list2 '
+            'are separated by separator, as are the items in the returned list.')
+
+    def evaluate(self, formatter, kwargs, mi, locals, list1, list2, separator):
+        l1 = [l.strip() for l in list1.split(separator) if l.strip()]
+        l2 = [icu_lower(l.strip()) for l in list2.split(separator) if l.strip()]
+
+        res = []
+        for i in l1:
+            if icu_lower(i) not in l2:
+                res.append(i)
+        return ', '.join(res)
+
+class BuiltinListIntersection(BuiltinFormatterFunction):
+    name = 'list_intersection'
+    arg_count = 3
+    category = 'List Manipulation'
+    __doc__ = doc = _('list_intersection(list1, list2, separator) -- '
+            'return a list made by removing from list1 any item not found in list2, '
+            'using a case-insensitive compare. The items in list1 and list2 '
+            'are separated by separator, as are the items in the returned list.')
+
+    def evaluate(self, formatter, kwargs, mi, locals, list1, list2, separator):
+        l1 = [l.strip() for l in list1.split(separator) if l.strip()]
+        l2 = [icu_lower(l.strip()) for l in list2.split(separator) if l.strip()]
+
+        res = []
+        for i in l1:
+            if icu_lower(i) in l2:
+                res.append(i)
+        return ', '.join(res)
+
+class BuiltinListSort(BuiltinFormatterFunction):
+    name = 'list_sort'
+    arg_count = 3
+    category = 'List Manipulation'
+    __doc__ = doc = _('list_sort(list, direction, separator) -- '
+            'return list sorted using a case-insensitive sort. If direction is '
+            'zero, the list is sorted ascending, otherwise descending. The list items '
+            'are separated by separator, as are the items in the returned list.')
+
+    def evaluate(self, formatter, kwargs, mi, locals, list1, direction, separator):
+        res = [l.strip() for l in list1.split(separator) if l.strip()]
+        return ', '.join(sorted(res, key=sort_key, reverse=direction != "0"))
+
+class BuiltinListEquals(BuiltinFormatterFunction):
+    name = 'list_equals'
+    arg_count = 6
+    category = 'List Manipulation'
+    __doc__ = doc = _('list_equals(list1, sep1, list2, sep2, yes_val, no_val) -- '
+            'return yes_val if list1 and list2 contain the same items, '
+            'otherwise return `no_val. The items are determined by splitting '
+            'each list using the appropriate separator character (sep1 or '
+            'sep2). The order of items in the lists is not relevant. '
+            'The compare is case insensitive.')
+
+    def evaluate(self, formatter, kwargs, mi, locals, list1, sep1, list2, sep2, yes_val, no_val):
+        s1 = set([icu_lower(l.strip()) for l in list1.split(sep1) if l.strip()])
+        s2 = set([icu_lower(l.strip()) for l in list2.split(sep2) if l.strip()])
+        if s1 == s2:
+            return yes_val
+        return no_val
 
 class BuiltinToday(BuiltinFormatterFunction):
     name = 'today'
@@ -799,51 +1016,65 @@ class BuiltinDaysBetween(BuiltinFormatterFunction):
         i = d1 - d2
         return str('%d.%d'%(i.days, i.seconds/8640))
 
+class BuiltinLanguageStrings(BuiltinFormatterFunction):
+    name = 'language_strings'
+    arg_count = 2
+    category = 'Get values from metadata'
+    __doc__ = doc = _('language_strings(lang_codes, localize) -- '
+            'return the strings for the language codes passed in lang_codes. '
+            'If localize is zero, return the strings in English. If '
+            'localize is not zero, return the strings in the language of '
+            'the current locale. Lang_codes is a comma-separated list.')
+    def evaluate(self, formatter, kwargs, mi, locals, lang_codes, localize):
+        retval = []
+        for c in [c.strip() for c in lang_codes.split(',') if c.strip()]:
+            try:
+                n = calibre_langcode_to_name(c, localize != '0')
+                if n:
+                    retval.append(n)
+            except:
+                pass
+        return ', '.join(retval)
 
-builtin_add         = BuiltinAdd()
-builtin_and         = BuiltinAnd()
-builtin_assign      = BuiltinAssign()
-builtin_booksize    = BuiltinBooksize()
-builtin_capitalize  = BuiltinCapitalize()
-builtin_cmp         = BuiltinCmp()
-builtin_contains    = BuiltinContains()
-builtin_count       = BuiltinCount()
-builtin_days_between= BuiltinDaysBetween()
-builtin_divide      = BuiltinDivide()
-builtin_eval        = BuiltinEval()
-builtin_first_non_empty = BuiltinFirstNonEmpty()
-builtin_field       = BuiltinField()
-builtin_format_date = BuiltinFormatDate()
-builtin_identifier_in_list = BuiltinIdentifierInList()
-builtin_ifempty     = BuiltinIfempty()
-builtin_in_list     = BuiltinInList()
-builtin_list_item   = BuiltinListitem()
-builtin_lookup      = BuiltinLookup()
-builtin_lowercase   = BuiltinLowercase()
-builtin_merge_lists = BuiltinMergeLists()
-builtin_multiply    = BuiltinMultiply()
-builtin_not         = BuiltinNot()
-builtin_ondevice    = BuiltinOndevice()
-builtin_or          = BuiltinOr()
-builtin_print       = BuiltinPrint()
-builtin_raw_field   = BuiltinRawField()
-builtin_re          = BuiltinRe()
-builtin_select      = BuiltinSelect()
-builtin_shorten     = BuiltinShorten()
-builtin_strcat      = BuiltinStrcat()
-builtin_strcmp      = BuiltinStrcmp()
-builtin_str_in_list = BuiltinStrInList()
-builtin_subitems    = BuiltinSubitems()
-builtin_sublist     = BuiltinSublist()
-builtin_substr      = BuiltinSubstr()
-builtin_subtract    = BuiltinSubtract()
-builtin_swaparound  = BuiltinSwapAroundComma()
-builtin_switch      = BuiltinSwitch()
-builtin_template    = BuiltinTemplate()
-builtin_test        = BuiltinTest()
-builtin_titlecase   = BuiltinTitlecase()
-builtin_today       = BuiltinToday()
-builtin_uppercase   = BuiltinUppercase()
+class BuiltinLanguageCodes(BuiltinFormatterFunction):
+    name = 'language_codes'
+    arg_count = 1
+    category = 'Get values from metadata'
+    __doc__ = doc = _('language_codes(lang_strings) -- '
+            'return the language codes for the strings passed in lang_strings. '
+            'The strings must be in the language of the current locale. '
+            'Lang_strings is a comma-separated list.')
+    def evaluate(self, formatter, kwargs, mi, locals, lang_strings):
+        retval = []
+        for c in [c.strip() for c in lang_strings.split(',') if c.strip()]:
+            try:
+                cv = canonicalize_lang(c)
+                if cv:
+                    retval.append(canonicalize_lang(cv))
+            except:
+                pass
+        return ', '.join(retval)
+
+_formatter_builtins = [
+    BuiltinAdd(), BuiltinAnd(), BuiltinAssign(), BuiltinBooksize(),
+    BuiltinCapitalize(), BuiltinCmp(), BuiltinContains(), BuiltinCount(),
+    BuiltinDaysBetween(), BuiltinDivide(), BuiltinEval(),
+    BuiltinFirstNonEmpty(), BuiltinField(), BuiltinFormatDate(),
+    BuiltinFormatNumber(), BuiltinFormatsModtimes(), BuiltinFormatsSizes(),
+    BuiltinHasCover(), BuiltinHumanReadable(), BuiltinIdentifierInList(),
+    BuiltinIfempty(), BuiltinLanguageCodes(), BuiltinLanguageStrings(),
+    BuiltinInList(), BuiltinListDifference(), BuiltinListEquals(),
+    BuiltinListIntersection(), BuiltinListitem(), BuiltinListSort(),
+    BuiltinListUnion(), BuiltinLookup(),
+    BuiltinLowercase(), BuiltinMultiply(), BuiltinNot(),
+    BuiltinOndevice(), BuiltinOr(), BuiltinPrint(), BuiltinRawField(),
+    BuiltinRe(), BuiltinSelect(), BuiltinShorten(), BuiltinStrcat(),
+    BuiltinStrcatMax(),
+    BuiltinStrcmp(), BuiltinStrInList(), BuiltinStrlen(), BuiltinSubitems(),
+    BuiltinSublist(),BuiltinSubstr(), BuiltinSubtract(), BuiltinSwapAroundComma(),
+    BuiltinSwitch(), BuiltinTemplate(), BuiltinTest(), BuiltinTitlecase(),
+    BuiltinToday(), BuiltinUppercase(),
+]
 
 class FormatterUserFunction(FormatterFunction):
     def __init__(self, name, doc, arg_count, program_text):
